#version 450

in vec2 uv;

uniform mat4 view;
uniform mat4 projection;
uniform mat4 projection_inverse;
uniform sampler2D albedo;
uniform sampler2D depth;
uniform sampler2D normals;

// SSAO
uniform sampler2D noiseTexture;

uniform SSAOSamples {
	vec3 samples[32];
};

uniform int kernel_size;
uniform float radius;
uniform float bias;

out float FragColor;

float CalculateSSAO(vec3 position, vec3 normal, vec2 texCoords)
{
	vec3 random_vec = vec3(texture(noiseTexture, texCoords * textureSize(depth, 0) / textureSize(noiseTexture, 0)).xy * 2.0 - 1.0, 0.0);
	vec3 tangent = normalize(random_vec - normal * dot(random_vec, normal));
	vec3 bitangent = cross(normal, tangent);
	mat3 tbn = mat3(tangent, bitangent, normal);

	float occlusion = 0.0;
	for(int i = 0; i < kernel_size; i++)
	{
		vec3 samplePos = tbn * samples[i];
		samplePos = position + samplePos * radius;

		vec4 offset = vec4(samplePos, 1.0);
		offset = projection * offset;
		offset.xyz /= offset.w;
		offset.xyz = offset.xyz * 0.5 + 0.5;

		vec4 temp = projection_inverse * vec4(offset.xy * 2. - 1., texture(depth, offset.xy).r * 2. - 1, 1.0);
		float sample_depth = (temp.xyz / temp.w).z;
		float range_check = smoothstep(0.0, 1.0, radius / abs(position.z - sample_depth));
		occlusion += (sample_depth >= samplePos.z + bias ? 1.0 : 0.0) * range_check;
	}
	
	return 1.0 - (occlusion / kernel_size);
}

void main() {
	vec4 temp = projection_inverse * vec4(uv.xy * 2. - 1., texture(depth, uv).r * 2. - 1, 1.0);
	vec3 position = temp.xyz / temp.w;
	vec3 normal = texture(normals, uv).xyz;
	vec3 albedo = texture(albedo, uv).rgb;

	FragColor = CalculateSSAO(position, normal, uv);
}